@@ -1,6 +1,6 @@
 ;;; psgml-edit.el --- Editing commands for SGML-mode with parsing support
 ;;
-;; $Id: psgml-edit.el,v 1.4 2000/10/06 08:38:12 youngs Exp $
+;; $Id: psgml-edit.el,v 1.6 2001/02/19 06:26:38 youngs Exp $
 
 ;; Copyright (C) 1994, 1995, 1996 Lennart Staflin
 
@@ -377,50 +377,49 @@ This uses the selective display feature."
   "Indent line, calling parser to determine level unless COL or ELEMENT
 is given.  If COL is given it should be the column to indent to.  If
 ELEMENT is given it should be a parse tree node, from which the level
-is determined.
-Deprecated: ELEMENT"
-  (sgml-debug "-> sgml-indent-line %s %s"
-              col (if element (sgml-element-gi element)))
+is determined."
   (when sgml-indent-step
-    (let ((here (point-marker))
-          ;; Where the indentation goes, i.e., will this be data
-          element-insert                
-          ;; Where we compute indentation, where the thing we indent is.
-          ;; Can be different from above if end-tag is omitted.
-          element-level)
+    (let ((here (point-marker)))
       (back-to-indentation)
-      (unless col
+      (unless (or col element)
 	;; Determine element
-	(setq element-insert
+	(setq element
 	      (let ((sgml-throw-on-error 'parse-error))
 		(catch sgml-throw-on-error
-                  ;; This used to be (sgml-find-element-of (point))
-                  ;; Why? Possibly to handle omitted end-tags
-                  (sgml-debug "-- sgml-indent-line find context")
-                  (sgml-find-context-of (point)))))
-        (setq element-level element-insert)
-        (when (and (not (eobp)) element-level)
-          (setq element-level (sgml-find-element-of (point)))
-          ;; It would be good if sgml-find-element-of would also tell
-          ;; us if the character is in the start-tag/end-tag or
-          ;; content
-          (when (or (= (point) (sgml-element-start element-level))
-                    (sgml-with-parser-syntax (sgml-is-end-tag)))
-            (setq element-level (sgml-element-parent element-level)))))
-      (when (eq element-level sgml-top-tree) ; not in a element at all
-	(setq element-level nil)        ; forget element
-	(goto-char here))		; insert normal tab insted)
-      (when element-level
-        (cond ((and (> (point) (sgml-element-start element-insert))
-                    (< (point) (sgml-element-stag-end element-insert))
-                    (not (sgml-element-data-p
-                          (sgml-element-parent element-insert))))
-               (setq col
-                     (funcall sgml-attribute-indent-function element-insert)))
-              ((or sgml-indent-data
-                   (not (sgml-element-data-p element-insert)))
-               (setq col
-                     (funcall sgml-content-indent-function element-level)))))
+		  (if (eobp)
+		      (sgml-find-context-of (point))
+		    (sgml-find-element-of (point)))))))
+      (when (eq element sgml-top-tree)	; not in a element at all
+	(setq element nil)		; forget element
+	(goto-char here))		; insert normal tab instead)
+      (when element
+	(sgml-with-parser-syntax
+	 (let ((stag (sgml-is-start-tag))
+	       (etag (sgml-is-end-tag)))
+           (cond ((and (> (point) (sgml-element-start element))
+                       (< (point) (sgml-element-stag-end element)))
+                  (setq col
+                        (+ (save-excursion
+                             (goto-char (sgml-element-start element))
+                             (current-column))
+                           (length (sgml-element-gi element))
+                           2)))
+                 ((and
+		   ;; Wing change, adapted by James
+		   (not (member* (sgml-element-gi
+				  (if (or stag etag)
+				      (sgml-element-parent element)
+				    element))
+				 sgml-inhibit-indent-tags :test #'equalp))
+		   (or sgml-indent-data
+		       (not (sgml-element-data-p
+			     (if stag
+				 (sgml-element-parent element)
+			       element))))
+		   (setq col
+			 (* sgml-indent-step
+			    (+ (if (or stag etag) -1 0)
+			       (sgml-element-level element))))))))))
       (when (and col (/= col (current-column)))
 	(beginning-of-line 1)    
 	(delete-horizontal-space)
